import 'dart:developer';
import 'dart:html';

import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:slick/configuration/slick_configuration.dart';
import 'package:slick/navigation/route_configuration.dart';
import 'package:slick/state/base_logic.dart';

class SlickScreen extends ConsumerWidget {
  @override
  Widget build(BuildContext context, ScopedReader watch) {
    currentRouteSingleton.building = true;
    var provider = currentRouteSingleton.logicForRoute();

    Widget bodyWidget;
    if (provider != null) {
      var baseLogic = watch(provider);
      if (baseLogic == null) return SlickConfiguration.unknownScreen;

      if (currentRouteSingleton.reInit) {
        baseLogic.deactivate();
      }

      if (currentRouteSingleton.reInit ||
          (!baseLogic.initialised && !baseLogic.ready && !baseLogic.hasError)) {
        logicInit(baseLogic);
      }

      if (baseLogic.hasError) {
        bodyWidget = SlickConfiguration.errorScreen;
      } else if (!baseLogic.ready) {
        print('Waiting');
        bodyWidget = SlickConfiguration.waitingScreen;
      } else {
        bodyWidget = currentRouteSingleton.screenForRoute();
      }
    } else {
      bodyWidget = currentRouteSingleton.screenForRoute();
    }

    currentRouteSingleton.building = false;

    return bodyWidget;
  }

  void logicInit(BaseLogic baseLogic) async {
    if (SlickConfiguration.isAppInitialised != null &&
        !SlickConfiguration.isAppInitialised.isCompleted)
      await SlickConfiguration.isAppInitialised.future;

    if (!baseLogic.hasViewAccess) {
      baseLogic.setError('You don\'t have permission for this route');
    } else {
      baseLogic.init();
      baseLogic.initialised = true;
      currentRouteSingleton.reInit = false;
    }
  }
}
